@@ -4,8 +4,10 @@ import copy
 
 from snack import SnackScreen
 
-from snackwich.patch import ListboxChoiceWindow, ButtonChoiceWindow, EntryWindow
+from snackwich.patch import ListboxChoiceWindow, ButtonChoiceWindow, \
+                            EntryWindow
 from snackwich.exceptions import GotoPanelException, BreakSuccessionException
+from snackwich.ui_functions import ProgressWindow
 
 class Snackwich(object):
     config = None
@@ -14,16 +16,65 @@ class Snackwich(object):
     # if there are any.
     cb_context = { }
 
+    # Which panels follows which (dictionary).
+    succession = None
+
+    # The first panel.
+    first_key = None
+
+    keys = None
+    keys_r = None
+
     # Some shorthand aliases to make the config nicer. These must be FROM'd 
     # into the current scope, above.
     aliases = { 'list':     ListboxChoiceWindow,
                 'choice':   ButtonChoiceWindow,
-                'entry':    EntryWindow
+                'entry':    EntryWindow,
+                'progress': ProgressWindow
               }
 
-    def __init__(self, config_filepath):
+    def __init__(self, config_filepath, forced_succession=None, 
+                 forced_first_key=None, config_overlap=None):
+        """
+        config_filepath: Panel configuration file-path.
+        forced_succession: An specific succession of which panels follow which 
+                           as an alternative to the automatically-calculated 
+                           one. Useful for multiple wizards comprised of a 
+                           subset of the windows having a custom order.
+        forced_first_key: A specific panel to start with instead of simply the 
+                          first one. Again, useful for wizards.
+        config_overlap: Dynamic values that will be written on top of the file-
+                        loaded values.
+        """
+
+        self.succession = forced_succession
+        self.first_key = forced_first_key
+    
         self.__read_config(config_filepath)
 
+        logging.info("Mapping panel succession.")
+
+        try:
+            (self.keys, self.keys_r) = self.__check_succession()
+        except:
+            logging.exception("Could not calculate panel succession.")
+            raise
+
+        if config_overlap:
+            for (key, expression) in config_overlap.iteritems():
+                if key not in self.keys_r:
+                    message = ("Can not overlap panel that does not already "
+                               "exist with name [%s]." % (key))
+
+                    logging.error(message)
+                    raise Exception(message)
+
+                for (expression_key, expression_value) \
+                        in expression.iteritems():
+                
+                    current_expression = self.config[self.keys_r[key]]
+                    current_expression[expression_key] = expression_value
+
     def __read_config(self, config_filepath):
         """Read and parse the given config file."""
     
@@ -161,16 +212,20 @@ class Snackwich(object):
             keys_r[key] = i
             i += 1
 
-        num_expressions = len(self.config)
-        succession = { }
-        i = 0
-        while i < num_expressions - 1:
-            succession[keys[i]] = keys[i + 1]
-            i += 1
+        if not self.succession:
+            num_expressions = len(self.config)
+            succession = { }
+            i = 0
+            while i < num_expressions - 1:
+                succession[keys[i]] = keys[i + 1]
+                i += 1
 
-        first_panel = keys[0] if succession else None
+            self.succession = succession
 
-        return (succession, first_panel, keys, keys_r)
+        if not self.first_key:
+            self.first_key = keys[0] if keys else None
+
+        return (keys, keys_r)
 
     def __slice_meta_attributes(self, dict_expression):
         meta_attributes = { }
@@ -188,18 +243,10 @@ class Snackwich(object):
     def execute(self):
         """Render each successive screen."""
 
-        logging.info("Mapping panel succession.")
-
-        try:
-            (succession, first_key, _, keys_r) = self.__check_succession()
-        except:
-            logging.exception("Could not calculate panel succession.")
-            raise
-
         results = { }
 
         # Make sure we actually had panels.
-        if not first_key:
+        if not self.first_key:
             return results;
 
         screen = SnackScreen()
@@ -208,12 +255,12 @@ class Snackwich(object):
             # Move through the panels in a linked-list fashion, so that we can 
             # reroute if told to.
         
-            key = first_key
+            key = self.first_key
             while 1:
                 logging.info("Processing panel expression with key [%s]." % 
                              (key))
 
-                index = keys_r[key]
+                index = self.keys_r[key]
                 expression = copy.deepcopy(self.config[index])
 
                 # Allow there to be a function call or lambda that must be 
@@ -237,7 +284,7 @@ class Snackwich(object):
                                      "to panel with key [%s]." % (key, 
                                                                   new_key))
                         
-                        if key not in keys_r:
+                        if key not in self.keys_r:
                             message = "We were told to go to panel with " \
                                       "invalid key [%s] while processing " \
                                       "panel with key [%s]." % (new_key, key)
@@ -284,11 +331,38 @@ class Snackwich(object):
                 try:                
                     result = self.__process_stanza(screen, key, expression, 
                                                    meta_attributes)
+
+                    if 'post_callback' in meta_attributes:
+                        callback = meta_attributes['post_callback']
                     
-                    if key in results:
-                        results[key].append(result)
+                        if not callable(callback):
+                            message = ("Callback for key [%s] is not "
+                                       "callable." % (key))
+                            
+                            logging.error(message)
+                            raise Exception(message)
+
+                        try:
+                            result = callback(key, result, expression)
+                        except BreakSuccessionException as e:
+                            logging.info("Post-callback for key [%s] has "
+                                         "instructed us to terminate." % (key))
+                            break
+                        except:
+                            logging.exception("An exception occured in the "
+                                              "post-callback for key [%s]." % 
+                                              (key))
+                            raise
+                    
+                    if 'collect_results' in meta_attributes and \
+                            meta_attributes['collect_results']:
+                        if key in results:
+                            results[key].append(result)
+                        else:
+                            results[key] = [result]
                     else:
-                        results[key] = [result]
+                        results[key] = result
+                            
                 except:
                     logging.exception("There was an exception while processing"
                                       " the stanza for key [%s]." % (key))
@@ -299,7 +373,7 @@ class Snackwich(object):
                 if 'next' in meta_attributes:
                     next_key = meta_attributes['next']
                     
-                    if key not in keys_r:
+                    if key not in self.keys_r:
                         logging.error("Key [%s] set as next from panel with "
                                       "key [%s] does not refer to a valid "
                                       "panel." % (key))
@@ -307,10 +381,10 @@ class Snackwich(object):
                     key = next_key
                 else:
                     # Go to the next panel, or break if no more.
-                    if key not in succession:
+                    if key not in self.succession:
                         break
 
-                    key = succession[key]
+                    key = self.succession[key]
         except:
             logging.exception("There was an exception while processing "
                               "config stanza with key [%s]." % (key))
