@@ -8,7 +8,8 @@ from snack import ButtonBar, TextboxReflowed, Listbox, GridFormHelp, Grid, \
 def ListboxChoiceWindow(screen, title, text, items, 
             buttons = ('Ok', 'Cancel'), 
             width = 40, scroll = 0, height = -1, default = None,
-            help = None, timer_ms = 0):
+            help = None, timer_ms = 0, secondary_message = None, 
+            secondary_message_width = None):
     """
     - ListboxChoiceWindow(screen, title, text, items, 
             buttons = ('Ok', 'Cancel'), 
@@ -42,10 +43,27 @@ def ListboxChoiceWindow(screen, title, text, items,
     if (default != None):
         l.setCurrent (default)
 
-    g = GridFormHelp(screen, title, help, 1, 3)
-    g.add(t, 0, 0)
-    g.add(l, 0, 1, padding = (0, 1, 0, 1))
-    g.add(bb, 0, 2, growx = 1)
+    g = GridFormHelp(screen, title, help, 1, 3 + (1 if secondary_message else 
+                                                  0))
+    
+    row = 0
+    
+    g.add(t, 0, row)
+    row += 1
+
+    if secondary_message:
+        if not secondary_message_width:
+            secondary_message_width = width
+    
+        t2 = TextboxReflowed(secondary_message_width, secondary_message)
+        g.add(t2, 0, row, padding = (0, 0, 0, 1))
+        row += 1
+
+    g.add(l, 0, row, padding = (0, 1, 0, 1))
+    row += 1
+    
+    g.add(bb, 0, row, growx = 1)
+    row += 1
 
     if timer_ms:
         g.form.w.settimer(timer_ms)
@@ -56,7 +74,8 @@ def ListboxChoiceWindow(screen, title, text, items,
 
 def ButtonChoiceWindow(screen, title, text, 
                buttons = [ 'Ok', 'Cancel' ], 
-               width = 40, x = None, y = None, help = None, timer_ms = 0):
+               width = 40, x = None, y = None, help = None, timer_ms = 0, 
+               secondary_message = None, secondary_message_width = None):
     """
      - ButtonChoiceWindow(screen, title, text, 
                buttons = [ 'Ok', 'Cancel' ], 
@@ -68,9 +87,24 @@ def ButtonChoiceWindow(screen, title, text,
     bb = ButtonBar(screen, buttons)
     t = TextboxReflowed(width, text, maxHeight = screen.height - 12)
 
-    g = GridFormHelp(screen, title, help, 1, 2)
-    g.add(t, 0, 0, padding = (0, 0, 0, 1))
-    g.add(bb, 0, 1, growx = 1)
+    g = GridFormHelp(screen, title, help, 1, 2 + (1 if secondary_message else 
+                                                  0))
+
+    row = 0
+
+    g.add(t, 0, row, padding = (0, 0, 0, 1))
+    row += 1
+
+    if secondary_message:
+        if not secondary_message_width:
+            secondary_message_width = width
+    
+        t2 = TextboxReflowed(secondary_message_width, secondary_message)
+        g.add(t2, 0, row, padding = (0, 0, 0, 1))
+        row += 1
+
+    g.add(bb, 0, row, growx = 1)
+    row += 1
 
     if timer_ms:
         g.form.w.settimer(timer_ms)
@@ -79,13 +113,12 @@ def ButtonChoiceWindow(screen, title, text,
 
 def EntryWindow(screen, title, text, prompts, allowCancel = 1, width = 40,
         entryWidth = 20, buttons = [ 'Ok', 'Cancel' ], help = None, 
-        timer_ms = 0):
-    """
-    EntryWindow(screen, title, text, prompts, allowCancel = 1, width = 40,
-        entryWidth = 20, buttons = [ 'Ok', 'Cancel' ], help = None):
-    """
+        timer_ms = 0, anchorLeft = 0, anchorRight = 1, 
+        secondary_message = None, secondary_message_width = None):
 
-    # Dustin: Added timer_ms parameter.
+    # Dustin: Added timer_ms parameter. Added anchorLeft and anchorRight as
+    # arguments to this function. Added secondary_message (test below primary 
+    # text).
 
     bb = ButtonBar(screen, buttons);
     t = TextboxReflowed(width, text)
@@ -106,16 +139,33 @@ def EntryWindow(screen, title, text, prompts, allowCancel = 1, width = 40,
         else:
             e = Entry(entryWidth)
 
-        sg.setField(Label(n), 0, count, padding = (0, 0, 1, 0), anchorLeft = 1)
+        sg.setField(Label(n), 0, count, padding=(0, 0, 1, 0), 
+                    anchorLeft=anchorLeft, anchorRight=anchorRight)
         sg.setField(e, 1, count, anchorLeft = 1)
         count = count + 1
         entryList.append(e)
 
-    g = GridFormHelp(screen, title, help, 1, 3)
+    g = GridFormHelp(screen, title, help, 1, 3 + (1 if secondary_message else 
+                                                  0))
+    
+    row = 0
 
-    g.add(t, 0, 0, padding = (0, 0, 0, 1))
-    g.add(sg, 0, 1, padding = (0, 0, 0, 1))
-    g.add(bb, 0, 2, growx = 1)
+    g.add(t, 0, row, padding = (0, 0, 0, 1))
+    row += 1
+
+    if secondary_message:
+        if not secondary_message_width:
+            secondary_message_width = width
+    
+        t2 = TextboxReflowed(secondary_message_width, secondary_message)
+        g.add(t2, 0, row, padding = (0, 0, 0, 1))
+        row += 1
+
+    g.add(sg, 0, row, padding = (0, 0, 0, 1))
+    row += 1
+    
+    g.add(bb, 0, row, growx = 1)
+    row += 1
 
     if timer_ms:
         g.form.w.settimer(timer_ms)
